package gregapi.code;

import gregapi.GT_API;
import gregapi.util.UT;

import java.util.*;

import net.minecraft.item.ItemStack;

/**
 * @author Gregorius Techneticies
 */
public class ItemStackSet<E extends ItemStackContainer> extends AbstractSet<E> {
	private transient HashMap<ItemStackContainer, Object> map;
    private static final Object OBJECT = new Object();
    
    public ItemStackSet() {
        map = new HashMap<ItemStackContainer, Object>();
        GT_API.STACKMAPS.add(map);
    }
    
    public ItemStackSet(Collection<? extends E> c) {
        map = new HashMap<ItemStackContainer, Object>(Math.max((int) (c.size()/.75f) + 1, 16));
        addAll(c);
        GT_API.STACKMAPS.add(map);
    }
    
    public ItemStackSet(int initialCapacity, float loadFactor) {
        map = new HashMap<ItemStackContainer, Object>(initialCapacity, loadFactor);
        GT_API.STACKMAPS.add(map);
    }
    
    public ItemStackSet(int initialCapacity) {
        map = new HashMap<ItemStackContainer, Object>(initialCapacity);
        GT_API.STACKMAPS.add(map);
    }
    
    ItemStackSet(int initialCapacity, float loadFactor, boolean dummy) {
        map = new LinkedHashMap<ItemStackContainer, Object>(initialCapacity, loadFactor);
        GT_API.STACKMAPS.add(map);
    }
    
    @Override
	public Iterator<E> iterator() {
        return (Iterator<E>)map.keySet().iterator();
    }
    
    @Override
	public int size() {
        return map.size();
    }
    
    @Override
	public boolean isEmpty() {
        return map.isEmpty();
    }
    
    @Override
	public boolean contains(Object aObject) {
    	if (aObject instanceof ItemStack) return map.containsKey(new ItemStackContainer((ItemStack)aObject));
        return map.containsKey(aObject);
    }
    
    public boolean add(ItemStack aStack) {
    	if (UT.Stacks.invalid(aStack)) return false;
        return map.put(new ItemStackContainer(aStack), OBJECT)==null;
    }
    
    @Override
	public boolean add(E e) {
        return map.put(e, OBJECT)==null;
    }
    
    @Override
	public boolean remove(Object o) {
        return map.remove(o)==OBJECT;
    }
    
    @Override
	public boolean equals(Object o) {
        return this == o;
    }
    
    @Override
	public void clear() {
        map.clear();
    }
}